@@ -3,6 +3,7 @@
 Note that this module is named 'built_ins' so as not to be confused with the
 special Python builtins module.
 """
+
 import atexit
 import builtins
 import collections.abc as cabc
@@ -18,8 +19,8 @@ import types
 import warnings
 from ast import AST
 
-from xonsh.inspectors import Inspector
-from xonsh.lazyasd import lazyobject
+from xonsh.lib.inspectors import Inspector
+from xonsh.lib.lazyasd import lazyobject
 from xonsh.platform import ON_POSIX
 from xonsh.tools import (
     XonshCalledProcessError,
@@ -52,15 +53,29 @@ def resetting_signal_handle(sig, f):
     """Sets a new signal handle that will automatically restore the old value
     once the new handle is finished.
     """
-    oldh = signal.getsignal(sig)
+    prev_signal_handler = signal.getsignal(sig)
 
-    def newh(s=None, frame=None):
+    def new_signal_handler(s=None, frame=None):
         f(s, frame)
-        signal.signal(sig, oldh)
+        signal.signal(sig, prev_signal_handler)
+        if sig == signal.SIGHUP:
+            """
+            SIGHUP means the controlling terminal has been lost. This should be
+            propagated to child processes so that they can decide what to do about it.
+            See also: https://www.gnu.org/software/bash/manual/bash.html#Signals
+            """
+            import xonsh.procs.jobs as xj
+
+            xj.hup_all_jobs()
         if sig != 0:
+            """
+            There is no immediate exiting here.
+            The ``sys.exit()`` function raises a ``SystemExit`` exception.
+            This exception must be caught and processed in the upstream code.
+            """
             sys.exit(sig)
 
-    signal.signal(sig, newh)
+    signal.signal(sig, new_signal_handler)
 
 
 def helper(x, name=""):
@@ -161,7 +176,7 @@ def pathsearch(func, s, pymode=False, pathobj=False):
 
 def subproc_captured_stdout(*cmds, envs=None):
     """Runs a subprocess, capturing the output. Returns the stdout
-    that was produced as a str.
+    that was produced as a str or list based on ``$XONSH_SUBPROC_OUTPUT_FORMAT``.
     """
 
     import xonsh.procs.specs
@@ -514,16 +529,33 @@ def xonsh_builtins(execer=None):
     XSH.unload()
 
 
+class InlineImporter:
+    """Inline importer allows to import and use module attribute or function in one line."""
+
+    def __getattr__(self, name):
+        if name.startswith("__"):
+            return getattr(super(), name)
+        return __import__(name)
+
+
 class XonshSession:
     """All components defining a xonsh session."""
 
     def __init__(self):
+        """
+        Attributes
+        ----------
+        exit: int or None
+            Session attribute. In case of integer value it signals xonsh to exit
+            with returning this value as exit code.
+        """
         self.execer = None
         self.ctx = {}
         self.builtins_loaded = False
         self.history = None
         self.shell = None
         self.env = None
+        self.imp = InlineImporter()
         self.rc_files = None
 
         # AST-invoked functions
@@ -560,6 +592,7 @@ class XonshSession:
         self._completers = None
         self.builtins = None
         self._initial_builtin_names = None
+        self.last = None  # Last executed CommandPipeline.
 
     @property
     def aliases(self):
@@ -592,7 +625,7 @@ class XonshSession:
         if self._py_quit is not None:
             builtins.quit = self._py_quit
 
-    def load(self, execer=None, ctx=None, **kwargs):
+    def load(self, execer=None, ctx=None, inherit_env=True, **kwargs):
         """Loads the session with default values.
 
         Parameters
@@ -601,6 +634,10 @@ class XonshSession:
             Xonsh execution object, may be None to start
         ctx : Mapping, optional
             Context to start xonsh session with.
+        inherit_env : bool
+            If ``True``: inherit environment variables from ``os.environ``.
+            If ``False``: use default values for environment variables and
+            set ``$XONSH_ENV_INHERITED = False``.
         """
         from xonsh.commands_cache import CommandsCache
         from xonsh.environ import Env, default_env
@@ -610,9 +647,14 @@ class XonshSession:
         if ctx is not None:
             self.ctx = ctx
 
-        self.env = kwargs.pop("env") if "env" in kwargs else Env(default_env())
+        if "env" in kwargs:
+            self.env = kwargs.pop("env")
+        elif inherit_env:
+            self.env = Env(default_env())
+        else:
+            self.env = Env({"XONSH_ENV_INHERITED": False})
 
-        self.exit = False
+        self.exit = None
         self.stdout_uncaptured = None
         self.stderr_uncaptured = None
 
@@ -676,6 +718,10 @@ class XonshSession:
 
         if not self.builtins_loaded:
             return
+
+        if self.history is not None:
+            self.history.flush(at_exit=True)
+
         self.unlink_builtins()
         delattr(builtins, "__xonsh__")
         self.builtins_loaded = False
