@@ -1,4 +1,5 @@
 """Tools to help interface with foreign shells, such as Bash."""
+
 import collections.abc as cabc
 import functools
 import os
@@ -10,7 +11,7 @@ import tempfile
 import warnings
 
 from xonsh.built_ins import XSH
-from xonsh.lazyasd import lazyobject
+from xonsh.lib.lazyasd import lazyobject
 from xonsh.platform import ON_CYGWIN, ON_MSYS, ON_WINDOWS
 from xonsh.tools import ensure_string, to_bool
 
