@@ -1,4 +1,5 @@
 """Command pipeline tools."""
+
 import io
 import os
 import re
@@ -8,9 +9,9 @@ import sys
 import threading
 import time
 
-import xonsh.jobs as xj
-import xonsh.lazyasd as xl
+import xonsh.lib.lazyasd as xl
 import xonsh.platform as xp
+import xonsh.procs.jobs as xj
 import xonsh.tools as xt
 from xonsh.built_ins import XSH
 from xonsh.procs.readers import ConsoleParallelReader, NonBlockingFDReader, safe_fdclose
@@ -28,7 +29,7 @@ def RE_HIDDEN_BYTES():
 
 @xl.lazyobject
 def RE_VT100_ESCAPE():
-    return re.compile(b"(\x9B|\x1B\\[)[0-?]*[ -\\/]*[@-~]")
+    return re.compile(b"(\x9b|\x1b\\[)[0-?]*[ -\\/]*[@-~]")
 
 
 @xl.lazyobject
@@ -61,6 +62,8 @@ def SIGNAL_MESSAGES():
 
 def safe_readlines(handle, hint=-1):
     """Attempts to read lines without throwing an error."""
+    if handle is None:
+        return []
     try:
         lines = handle.readlines(hint)
     except OSError:
@@ -92,23 +95,27 @@ class CommandPipeline:
     """Represents a subprocess-mode command pipeline."""
 
     attrnames = (
-        "stdin",
-        "stdout",
-        "stderr",
-        "pid",
         "returncode",
+        "suspended",
+        "pid",
         "args",
         "alias",
-        "stdin_redirect",
-        "stdout_redirect",
-        "stderr_redirect",
-        "timestamps",
         "executed_cmd",
+        "timestamps",
         "input",
         "output",
         "errors",
     )
 
+    attrnames_ext = (
+        "stdin",
+        "stdout",
+        "stderr",
+        "stdin_redirect",
+        "stdout_redirect",
+        "stderr_redirect",
+    )
+
     nonblocking = (io.BytesIO, NonBlockingFDReader, ConsoleParallelReader)
 
     def __init__(self, specs):
@@ -149,6 +156,8 @@ class CommandPipeline:
         self._raw_output = self._raw_error = b""
         self._stderr_prefix = self._stderr_postfix = None
         self.term_pgid = None
+        self.suspended = None
+        self.output_format = self.spec.output_format
 
         background = self.spec.background
         pipeline_group = None
@@ -159,7 +168,9 @@ class CommandPipeline:
             # taking the terminal away from the `less` command, causing `less`
             # to stop.
             pipeline_group = os.getpgid(0)
-        for spec in specs:
+        for i, spec in enumerate(specs):
+            for mod in spec.decorators:
+                mod.decorate_spec_pre_run(self, spec, i)
             if self.starttime is None:
                 self.starttime = time.time()
             try:
@@ -182,8 +193,14 @@ class CommandPipeline:
         self.proc = self.procs[-1]
 
     def __repr__(self):
+        debug = XSH.env.get("XONSH_DEBUG", False)
+        attrs = self.attrnames + (self.attrnames_ext if debug else ())
         s = self.__class__.__name__ + "(\n  "
-        s += ",\n  ".join(a + "=" + repr(getattr(self, a)) for a in self.attrnames)
+        s += ",\n  ".join(
+            a + "=" + repr(getattr(self, a))
+            for a in attrs
+            if debug or getattr(self, a) is not None
+        )
         s += "\n)"
         return s
 
@@ -250,7 +267,10 @@ class CommandPipeline:
             # we get here if the process is not threadable or the
             # class is the real Popen
             PrevProcCloser(pipeline=self)
-            task = xj.wait_for_active_job()
+            task = None
+            if not isinstance(sys.exc_info()[1], SystemExit):
+                task = xj.wait_for_active_job()
+
             if task is None or task["status"] != "stopped":
                 proc.wait()
                 self._endtime()
@@ -261,7 +281,7 @@ class CommandPipeline:
                     lines = b.splitlines(keepends=True)
                     yield from lines
                     self.end(tee_output=False)
-                elif self.captured == "stdout":
+                elif self.captured == "stdout" and stdout is not None:
                     b = stdout.read()
                     s = self._decode_uninew(b, universal_newlines=True)
                     self.lines = s.splitlines(keepends=True)
@@ -281,7 +301,9 @@ class CommandPipeline:
         prev_end_time = None
         i = j = cnt = 1
         while proc.poll() is None:
-            if getattr(proc, "suspended", False):
+            if getattr(proc, "suspended", False) or self._procs_suspended() is not None:
+                self.suspended = True
+                xj.update_job_attr(proc.pid, "status", "suspended")
                 return
             elif getattr(proc, "in_alt_mode", False):
                 time.sleep(0.1)  # probably not leaving any time soon
@@ -296,6 +318,7 @@ class CommandPipeline:
                 self._close_prev_procs()
                 proc.prevs_are_closed = True
                 break
+
             stdout_lines = safe_readlines(stdout, 1024)
             i = len(stdout_lines)
             if i != 0:
@@ -506,6 +529,20 @@ class CommandPipeline:
     def _safe_close(self, handle):
         safe_fdclose(handle, cache=self._closed_handle_cache)
 
+    def _procs_suspended(self):
+        """Check procs and return suspended proc."""
+        for proc in self.procs:
+            info = xj.proc_untraced_waitpid(proc, hang=False)
+            if getattr(proc, "suspended", False):
+                proc = getattr(proc, "proc", proc)
+                procname = f"{getattr(proc, 'args', '')} with pid {proc.pid}".strip()
+                print(
+                    f"Process {procname} was suspended with signal {info['signal_name']} and placed in `jobs`.\n"
+                    f"This happens when a process starts waiting for input but there is no terminal attached in captured mode.",
+                    file=sys.stderr,
+                )
+                return proc
+
     def _prev_procs_done(self):
         """Boolean for if all previous processes have completed. If there
         is only a single process in the pipeline, this returns False.
@@ -595,14 +632,21 @@ class CommandPipeline:
         spec = self.spec
         rtn = self.returncode
 
-        if rtn is None or rtn == 0 or not XSH.env.get("RAISE_SUBPROC_ERROR"):
+        if rtn is None or rtn == 0:
             return
 
-        try:
-            raise subprocess.CalledProcessError(rtn, spec.args, output=self.output)
-        finally:
-            # this is need to get a working terminal in interactive mode
-            self._return_terminal()
+        raise_subproc_error = spec.raise_subproc_error
+        if callable(raise_subproc_error):
+            raise_subproc_error = raise_subproc_error(spec, self)
+        if raise_subproc_error is False:
+            return
+
+        if raise_subproc_error or XSH.env.get("RAISE_SUBPROC_ERROR", True):
+            try:
+                raise subprocess.CalledProcessError(rtn, spec.args, output=self.output)
+            finally:
+                # this is need to get a working terminal in interactive mode
+                self._return_terminal()
 
     #
     # Properties
@@ -628,15 +672,33 @@ class CommandPipeline:
         """Creates normalized input string from args."""
         return " ".join(self.args)
 
+    def get_formatted_lines(self, lines):
+        """Format output lines."""
+        fmt = self.output_format
+        if fmt == "stream_lines":
+            if len(lines) == 1:
+                return lines[0].rstrip("\n")
+            else:
+                return "".join(lines)
+        elif fmt == "list_lines":
+            if not lines:
+                return lines
+            elif len(lines) == 1:
+                return [lines[0].rstrip("\n")]
+            else:
+                return [line.rstrip("\n") for line in lines]
+        elif callable(fmt):
+            return fmt(lines)
+
     @property
     def output(self):
         """Non-blocking, lazy access to output"""
         if self.ended:
             if self._output is None:
-                self._output = "".join(self.lines)
+                self._output = self.get_formatted_lines(self.lines)
             return self._output
         else:
-            return "".join(self.lines)
+            return self.get_formatted_lines(self.lines)
 
     @property
     def out(self):
