@@ -31,6 +31,7 @@ import os
 import pathlib
 import re
 import shlex
+import signal
 import string
 import subprocess
 import sys
@@ -40,10 +41,23 @@ import typing as tp
 import warnings
 from contextlib import contextmanager
 
+try:
+    from prompt_toolkit.cursor_shapes import (
+        CursorShape,
+        CursorShapeConfig,
+        DynamicCursorShapeConfig,
+        ModalCursorShapeConfig,
+        SimpleCursorShapeConfig,
+    )
+
+    HAVE_CURSOR_SHAPE = True
+except ImportError:
+    HAVE_CURSOR_SHAPE = False
+
 # adding imports from further xonsh modules is discouraged to avoid circular
 # dependencies
 from xonsh import __version__
-from xonsh.lazyasd import LazyDict, LazyObject, lazyobject
+from xonsh.lib.lazyasd import LazyDict, LazyObject, lazyobject
 from xonsh.platform import (
     DEFAULT_ENCODING,
     HAS_PYGMENTS,
@@ -56,8 +70,11 @@ from xonsh.platform import (
 
 
 @contextmanager
-def chdir(adir):
+def chdir(adir, mkdir=False):
+    adir = pathlib.Path(adir)
     old_dir = os.getcwd()
+    if mkdir:
+        os.makedirs(adir, exist_ok=True)
     os.chdir(adir)
     try:
         yield
@@ -208,8 +225,7 @@ class EnvPath(cabc.MutableSequence):
                 self._l = args
             else:
                 raise TypeError(
-                    "EnvPath cannot be initialized with items "
-                    "of type %s" % type(args)
+                    f"EnvPath cannot be initialized with items of type {type(args)}: {args!r}"
                 )
 
     def __getitem__(self, item):
@@ -228,8 +244,24 @@ class EnvPath(cabc.MutableSequence):
     def __delitem__(self, key):
         self._l.__delitem__(key)
 
+    @staticmethod
+    def _prepare_path(p):
+        return str(expand_path(p))
+
     def insert(self, index, value):
-        self._l.insert(index, value)
+        self._l.insert(index, self._prepare_path(value))
+
+    def append(self, value):
+        self._l.append(self._prepare_path(value))
+
+    def prepend(self, value):
+        self._l.insert(0, self._prepare_path(value))
+
+    def remove(self, value):
+        try:
+            self._l.remove(self._prepare_path(value))
+        except ValueError:
+            print(f"EnvPath warning: path {repr(value)} not found.", file=sys.stderr)
 
     @property
     def paths(self):
@@ -292,7 +324,7 @@ class EnvPath(cabc.MutableSequence):
         None
 
         """
-        data = str(expand_path(data))
+        data = self._prepare_path(data)
         if data not in self._l:
             self._l.insert(0 if front else len(self._l), data)
         elif replace:
@@ -806,62 +838,6 @@ class redirect_stderr(_RedirectStream):
     _stream = "stderr"
 
 
-def _yield_accessible_unix_file_names(path):
-    """yield file names of executable files in path."""
-    if not os.path.exists(path):
-        return
-    for file_ in os.scandir(path):
-        try:
-            if file_.is_file() and os.access(file_.path, os.X_OK):
-                yield file_.name
-        except OSError:
-            # broken Symlink are neither dir not files
-            pass
-
-
-def _executables_in_posix(path):
-    if not os.path.exists(path):
-        return
-    else:
-        yield from _yield_accessible_unix_file_names(path)
-
-
-def _executables_in_windows(path):
-    if not os.path.isdir(path):
-        return
-    extensions = xsh.env["PATHEXT"]
-    try:
-        for x in os.scandir(path):
-            try:
-                is_file = x.is_file()
-            except OSError:
-                continue
-            if is_file:
-                fname = x.name
-            else:
-                continue
-            base_name, ext = os.path.splitext(fname)
-            if ext.upper() in extensions:
-                yield fname
-    except FileNotFoundError:
-        # On Windows, there's no guarantee for the directory to really
-        # exist even if isdir returns True. This may happen for instance
-        # if the path contains trailing spaces.
-        return
-
-
-def executables_in(path) -> tp.Iterable[str]:
-    """Returns a generator of files in path that the user could execute."""
-    if ON_WINDOWS:
-        func = _executables_in_windows
-    else:
-        func = _executables_in_posix
-    try:
-        yield from func(path)
-    except PermissionError:
-        return
-
-
 def debian_command_not_found(cmd):
     """Uses the debian/ubuntu command-not-found utility to suggest packages for a
     command that cannot currently be found.
@@ -1008,8 +984,8 @@ def print_warning(msg):
     sys.stderr.write(msg)
 
 
-def print_exception(msg=None, exc_info=None):
-    """Print given exception (or current if None) with/without traceback and set sys.last_type, sys.last_value, sys.last_traceback accordingly."""
+def print_exception(msg=None, exc_info=None, source_msg=None):
+    """Print given exception (or current if None) with/without traceback and set sys.last_exc accordingly."""
 
     # is no exec_info() triple is given, use the exception beeing handled at the moment
     if exc_info is None:
@@ -1039,7 +1015,17 @@ def print_exception(msg=None, exc_info=None):
         limit = 0
         chain = False
 
-    sys.last_type, sys.last_value, sys.last_traceback = exc_info
+    if xsh.env.get("XONSH_SHOW_TRACEBACK", False):
+        """
+        This moved under ``XONSH_SHOW_TRACEBACK`` because it looks that python's
+        internal machinery behind ``sys.last_*`` is not thread safe
+        when traceback is not printed (#5408).
+        """
+        if sys.version_info >= (3, 12):
+            # https://docs.python.org/3/library/sys.html#sys.last_exc
+            sys.last_exc = exc_info
+        else:
+            sys.last_type, sys.last_value, sys.last_traceback = exc_info
 
     manually_set_trace, show_trace = _get_manual_env_var("XONSH_SHOW_TRACEBACK", False)
     manually_set_logfile, log_file = _get_manual_env_var("XONSH_TRACEBACK_LOGFILE")
@@ -1052,6 +1038,9 @@ def print_exception(msg=None, exc_info=None):
     # convert show_trace to bool if necessary
     if not is_bool(show_trace):
         show_trace = to_bool(show_trace)
+
+    if source_msg:
+        sys.stderr.write(source_msg + "\n")
     # if the trace option has been set, print all traceback info to stderr
     if show_trace:
         # notify user about XONSH_TRACEBACK_LOGFILE if it has
@@ -1080,8 +1069,8 @@ def print_exception(msg=None, exc_info=None):
             "RAISE_SUBPROC_ERROR"
         ):
             display_colored_error_message(exc_info, limit=1)
-            return
-        display_error_message(exc_info)
+        else:
+            display_error_message(exc_info)
     if msg:
         msg = msg if msg.endswith("\n") else msg + "\n"
         sys.stderr.write(msg)
@@ -1097,19 +1086,16 @@ def display_colored_error_message(exc_info, strip_xonsh_error_types=True, limit=
 
     content = traceback.format_exception(*exc_info, limit=limit)
 
-    if (
-        no_trace_and_raise_subproc_error
-        and "subprocess.CalledProcessError:" in content[-1]
-    ):
-        content = content[:-1]
+    if no_trace_and_raise_subproc_error and "Error:" in content[-1]:
+        content = [content[-1]]
 
     traceback_str = "".join([v for v in content])
-    traceback_str += "" if traceback_str.endswith("\n") else "\n"
 
     # color the traceback if available
     _, interactive = _get_manual_env_var("XONSH_INTERACTIVE", 0)
     _, color_results = _get_manual_env_var("COLOR_RESULTS", 0)
-    if not interactive or not color_results or not HAS_PYGMENTS:
+    shell_loaded = hasattr(xsh.shell, "shell")  # do not color traceback in xonshrc file
+    if not interactive or not color_results or not shell_loaded or not HAS_PYGMENTS:
         sys.stderr.write(traceback_str)
         return
 
@@ -1118,7 +1104,7 @@ def display_colored_error_message(exc_info, strip_xonsh_error_types=True, limit=
     lexer = pygments.lexers.python.PythonTracebackLexer()
     tokens = list(pygments.lex(traceback_str, lexer=lexer))
     # this goes to stdout, but since we are interactive it doesn't matter
-    print_color(tokens, end="\n", file=sys.stderr)
+    print_color(tokens, end="", file=sys.stderr)
     return
 
 
@@ -1755,6 +1741,48 @@ def ptk2_color_depth_setter(x):
     return x
 
 
+def ptk_cursor_shape_vi_modal():
+    if xsh.env.get("VI_MODE"):
+        return ModalCursorShapeConfig()
+    else:
+        return SimpleCursorShapeConfig()
+
+
+def to_ptk_cursor_shape(x):
+    if not HAVE_CURSOR_SHAPE:
+        return None
+    if isinstance(x, (CursorShape, CursorShapeConfig)):
+        return x
+    if not isinstance(x, str):
+        raise ValueError("invalid cursor shape")
+    x = str(x).upper().replace("-", "_")
+    if x == "MODAL":
+        return ModalCursorShapeConfig()
+    elif x == "MODAL_VI_MODE_ONLY":
+        return DynamicCursorShapeConfig(ptk_cursor_shape_vi_modal)
+    try:
+        return CursorShape[x]
+    except KeyError:
+        return SimpleCursorShapeConfig()
+
+
+def to_ptk_cursor_shape_display_value(x):
+    if not x:
+        return ""
+    if isinstance(x, SimpleCursorShapeConfig):
+        x = x.get_cursor_shape(None)
+    if isinstance(x, CursorShape):
+        x = x.value.lower().replace("_", "-")
+        if x.startswith("-"):
+            x = x[1:]
+        return x
+    if isinstance(x, ModalCursorShapeConfig):
+        return "modal"
+    if isinstance(x, DynamicCursorShapeConfig):
+        return "modal-vi-mode-only"
+    return "unknown"
+
+
 def is_completions_display_value(x):
     """Enumerated values of ``$COMPLETIONS_DISPLAY``"""
     return x in {"none", "single", "multi"}
@@ -1791,7 +1819,9 @@ def to_completion_mode(x):
     y = (
         "default"
         if y in ("", "d", "xonsh", "none", "def")
-        else "menu-complete" if y in ("m", "menu", "menu-completion") else y
+        else "menu-complete"
+        if y in ("m", "menu", "menu-completion")
+        else y
     )
     if y not in CANONIC_COMPLETION_MODES:
         warnings.warn(
@@ -1806,7 +1836,7 @@ def to_completion_mode(x):
 def is_tok_color_dict(x):
     from pygments.token import _TokenType, string_to_tokentype
 
-    from xonsh.ptk_shell.shell import _style_from_pygments_dict
+    from xonsh.shells.ptk_shell import _style_from_pygments_dict
 
     """Tests if something is a Token:Style dictionary"""
     if not isinstance(x, dict):
@@ -2046,11 +2076,14 @@ def format_color(string, **kwargs):
         return ansi_partial_color_format(string, style=style)
 
 
-def print_color(string, **kwargs):
+def print_color(string, color=None, **kwargs):
     """Prints a string that may contain colors. This dispatched to the shell
     method of the same name. Colors will be formatted if they have not already
     been.
     """
+    if color:
+        string = f"{{{color.upper()}}}{string}{{RESET}}"
+
     if hasattr(xsh.shell, "shell"):
         xsh.shell.shell.print_color(string, **kwargs)
     else:
@@ -2820,3 +2853,39 @@ def to_repr_pretty_(inst, p, cycle):
         elif len(inst):
             p.break_()
             p.pretty(dict(inst))
+
+
+def get_signal_name(signum):
+    """Return a signal name by the signal number."""
+    for name in dir(signal):
+        if name.startswith("SIG") and getattr(signal, name) == signum:
+            return name
+    return ""
+
+
+def describe_waitpid_status(status):
+    """Describes ``pid, status = os.waitpid(pid, opt)`` status."""
+    funcs = [
+        os.WIFEXITED,
+        os.WEXITSTATUS,
+        os.WIFSIGNALED,
+        os.WTERMSIG,
+        os.WIFSTOPPED,
+        os.WSTOPSIG,
+        os.WCOREDUMP,
+    ]
+    for f in funcs:
+        s = f(status)
+        print(f.__name__, "-", s, get_signal_name(s), "-", f.__doc__)
+
+
+def unquote(s: str, chars="'\""):
+    """Strip paired quotes from string once."""
+    if len(s) >= 2 and s[0] == s[-1] and s[0] in chars:
+        return s[1:-1]
+    return s
+
+
+def endswith_newline(s: str):
+    """Force one new line character end to string."""
+    return s.rstrip("\n") + "\n"
